/**
 * The main file that renders a visualisation of circuits in DNA.
 */

var data = {
    nodes: [],
    edges: [],
    min_position: 1111111111111111111,
    max_position: 0,
    max_dist: 0,
};

var threeD_attr = set_up_3D_attr();

var dna_attr = {
    rect: {
        node_points: [],
        colour_outer: "black",
        colour_inner: "green",
        width: 3,
        height: 5,
    },
    semi_circle: {
        outer_radius: 100, // height, width = 2 * radius
        inner_radius: 90,
        stroke_colour: "white",
        detailed_radi: {},
        rad_types: "normal", // is either normal for an even spread or multi for
        // radi dependent on num edges
    },
    main_strand: {
        stroke_width:  1,
        stroke_colour: "black",//"#627071", // nearly black
        tail_length: 50
    },
    dna_strand: {
        stroke_width: 1,
        internal_dist: 50,
        external_dist: 200, // used to set a limit for how tall the strand is
        dist_stroke_colour: "#627071"
    },
    vis: {
        scale: {},
        width: 1200,
        height: 400,
        fdg_rad: 5,
    }
};

/**
 *
 * Parses the main json file containing information
 * regarding the nodes and edges of the file.
 */
var run_dna_visualisation = function () {
    dna_attr.node_points = [];
    var nodes = dna_attr.data['nodes'];
    var edges = dna_attr.data['edges'];
    nodes = clean_json(nodes);
    edges = clean_json(edges);
    divide_edge_dict(edges, nodes);
    var ordered_nodes = order_nodes(nodes);
    data['ordered_nodes'] = ordered_nodes;

    dna_attr.rect.colour_inner =  d3.scaleLinear().domain([1,ordered_nodes.length])
      .interpolate(d3.interpolateHcl)
      .range([d3.rgb("#007AFF"), d3.rgb('#FFF500')]);
    dna_attr.rect. colour_outer = d3.scaleOrdinal(d3.schemeCategory20b);

    data.edges = edges;
    get_max_min(nodes, edges);
    setup_strand_svg();

    // Setup the groups for the network
    run_network_update();
    draw_network_axis();
    draw_nodes(ordered_nodes);
    draw_dna_len(ordered_nodes);

    draw_interaction_points();
    draw_other_networks();
    run_fdg(make_into_fdg_format(data.nodes, data.from_edges, data.to_edges));

};

var make_into_fdg_format = function (nodes, edges, to_edges) {
    var fdg = {};
    var fdg_nodes = [];
    var placed_nodes = {};
    for (var n in nodes) {
        var tmp = {};
        tmp['id'] = nodes[n].actual_id;
        tmp['name'] = n;
        if (nodes[n].next === undefined) {
            tmp['len_to_next'] = 0;
        } else {
            tmp['len_to_next'] = ((nodes[n].next.start - nodes[n].start) / data.max_dist);
        }
        var id = nodes[n].actual_id;
        if (edges[n] != undefined) {
            if (edges[n].length > 2) {
                tmp['group'] = id.substr(1) + "-" + 1;
                tmp['opacity'] = 1;
            } else if (edges[n].length > 1) {
                tmp['group'] = id.substr(1) + "-" + 0.7;
                tmp['opacity'] = 0.7;
            } else {
                tmp['group'] = id.substr(1) + "-" + 0.2;
                tmp['opacity'] = 0.2;
            }
        } else {
            if (to_edges[n].length > 2) {
                tmp['group'] = id.substr(1) + "-" + 1;
                tmp['opacity'] = 1;
            } else if (to_edges[n].length > 1) {
                tmp['group'] = id.substr(1) + "-" + 0.7;
                tmp['opacity'] = 0.7;
            } else {
                tmp['group'] = id.substr(1) + "-" + 0.2;
                tmp['opacity'] = 0.2;
            }

        }
        tmp['colour'] = dna_attr.rect.colour_inner(nodes[n].actual_id.substr(1) % 20);
        tmp['stroke-colour'] = dna_attr.rect.colour_outer(nodes[n].chrom);
        fdg_nodes.push(tmp);
        placed_nodes[nodes[n].actual_id.substr(1)] = tmp;
    }
    var fdg_edges = [];
    for (var e in edges) {
        for (var edge in edges[e]) {
            var tmp = {};
            tmp['source'] = nodes[edges[e][edge].from].actual_id;
            tmp['target'] = nodes[edges[e][edge].to].actual_id;
            if (edges[e].length > 2) {
                tmp['opacity'] = ``;
                tmp['value'] = nodes[edges[e][edge].from].actual_id.substr(1) + "-" + 1;
            } else if (edges[e].length > 1) {
                tmp['value'] = nodes[edges[e][edge].from].actual_id.substr(1) + "-" + 0.5;
                tmp['opacity'] = 0.7;
            } else {
                tmp['value'] = nodes[edges[e][edge].from].actual_id.substr(1) + "-" + 0.1;
                tmp['opacity'] = 0.2;
            }
            tmp['colour'] = dna_attr.rect.colour_inner(nodes[edges[e][edge].from].actual_id.substr(1) % 20);
            fdg_edges.push(tmp);
        }
    }
    fdg['nodes'] = fdg_nodes;
    fdg['links'] = fdg_edges;
    data['links'] = fdg_edges;
    data['placed_nodes'] = fdg_nodes;
    data['placed_node_dict'] = placed_nodes;
    return fdg;
}

/**
 *
 * Helper that changes the JSON object from being an array
 * to a string or int (i.e. makes the first element of the object
 * the only element.)
 */
var clean_json = function (values) {
    var clean_values = {};
    for (var v in values[0]) {
        var cleaned = {};
        for (var v_inner in values[0][v]) {
            cleaned[v_inner] = values[0][v][v_inner][0];
        }
        clean_values[values[0][v].label] = cleaned;
    }
    return clean_values;
};


/**
 * Sort the nodes descending based on the number of edges.
 */
var sort_nodes = function (nodes) {
    nodes.sort(function (a, b) {
        return a.start - b.start;
    });
    return nodes;
};

/**
 * Orders the nodes by their start position
 */
var order_nodes = function (nodes) {
    var ordered_nodes = [];
    var pi = Math.PI;
    var full_circle = pi;
    var region_len = full_circle / (Object.keys(nodes).length);
    for (var i in nodes) {
        // Below is for the 3D FDG and rendering that
        nodes[i]['x'] = 0;
        nodes[i]['y'] = 0;
        nodes[i]['x_y'] = {'x': 0, 'y': 0};
        ordered_nodes.push(nodes[i]);
    }
    // Want to calculate the maximum distance between two nodes.
    ordered_nodes = sort_nodes(ordered_nodes);
    for (var n = 0; n < ordered_nodes.length; n++) {
        ordered_nodes[n]['actual_id'] = 'n' + n;
        nodes[ordered_nodes[n].label]['actual_id'] = 'n' + n;
        /**
         *
         * Below is for drawing the DNA strand
         */
        var start_angle = (n * region_len);
        var end_angle = (n * region_len);
        var next_node = ordered_nodes[n + 1];
        // Store the next node in the dictionary
        nodes[ordered_nodes[n].label]['next'] = next_node;
        var this_node = ordered_nodes[n];
        if (next_node != undefined) {
            var node_diff = next_node.start - this_node.start;
            if (node_diff > data.max_dist) {
                data.max_dist = ordered_nodes[n + 1].start - ordered_nodes[n].start;
            }
        }
        // Save the start and end angle to the node
        ordered_nodes[n].start_angle = start_angle;
        ordered_nodes[n].end_angle = end_angle;
    }
    data.nodes = nodes;
    return ordered_nodes;
}

/**
 *
 * Sorts edges into two dictionaries.
 * One has: to -> and
 * the other: from ->
 * nodes as the keys.
 */
var divide_edge_dict = function (edges, nodes) {
    var to_edges = {};
    var from_edges = {};
    var max_out_edge = 0;
    for (var e in edges) {
        if (to_edges[edges[e].to] == undefined) {
            to_edges[edges[e].to] = [];
        }
        to_edges[edges[e].to].push(edges[e]);
        if (from_edges[edges[e].from] == undefined) {
            from_edges[edges[e].from] = [];
        }
        from_edges[edges[e].from].push(edges[e]);
    }
    for (var e in from_edges) {
        var edge = from_edges[e];
        nodes[e]['num_out_edge'] = edge.length;
        if (edge.length > max_out_edge) {
            max_out_edge = edge.length;
        }
    }
    for (var e in to_edges) {
        var edge = to_edges[e];
        nodes[e]['num_in_edge'] = edge.length;
    }
    // Want to set up different radi for each of the edge lengths
    dna_attr.semi_circle.detailed_radi = {};
    for (var i = 0; i < max_out_edge; i ++) {
        dna_attr.semi_circle.detailed_radi[i] = dna_attr.semi_circle.outer_radius * ((i + 1) / max_out_edge);
    }

    data['from_edges'] = from_edges;
    data['to_edges'] = to_edges;
}

/**
 *
 * Gets the max and min of the position.
 */
var get_max_min = function (nodes) {
    for (var n in nodes) {
        var node = nodes[n];
        if (node.end > data.max_position) {
            data.max_position = node.end;
        }
        if (node.start < data.min_position) {
            data.min_position = node.start;
        }
    }
}


function getPosition(force) {
    return force.nodes().map(function (node) {
        return {
            id: node.id,
            name: node.name,
            group: node.group,
            x: node.x,
            y: node.y
        }
    })
}

var run_fdg = function (graph) {
     var       margin = {top: 0, right: 0, bottom: 50, left: 10};

    var svg = d3.select("svg#interactionvis"),
            width = +svg.attr("width"),
            height = +svg.attr("height")

    // Remove all existing content
    svg.selectAll("*").remove();

    svg =  svg.append("g")
             .attr("transform", "translate(" + margin.left +
                    "," + margin.top + ")");
    dna_attr.fdg_svg = svg;

    var simulation = d3.forceSimulation()
            .force("link", d3.forceLink().id(function (d) {
                return d.id;
            }))
            .force("charge", d3.forceManyBody())
            .force("center", d3.forceCenter(width / 2, height / 2))
            .on('end', function () {
                // Update the thread vis
                draw_3D_nodes(data['placed_nodes'], threeD_attr);
            });


    var link = svg.append("g")
            .attr("class", "links")
            .selectAll("line")
            .data(graph.links)
            .enter().append("line")
            .attr("stroke", function (d) {
                return d.colour;
            })
            .attr("opacity", function (d) {
                return d.value.split("-")[1];
            })
            .attr("stroke-width", 2);

    var node = svg.append("g")
            .attr("class", "nodes")
            .selectAll("circle")
            .data(graph.nodes)
            .enter().append("circle")
            .attr("id", function(d) {
                return "fdg-" + d.id;
    })
            .attr("r", dna_attr.vis.fdg_rad)
            .attr("class", "nodes")
            .attr("fill", function (d) {
                return d.colour;
            })
            .attr("opacity", function (d) {
                return 1;//d.group.split("-")[1];
            })

            .call(d3.drag()
                    .on("start", dragstarted)
                    .on("drag", dragged)
                    .on("end", dragended))
        .on("mouseover", function(d) {

            dna_attr.vis.network_grp.select("#interaction-" + d.id).attr("fill", "black");
            dna_attr.vis.network_grp.select("#interaction-" + d.id).attr("opacity", 1);
            dna_attr.vis.network_grp.select("#interaction-" + d.id).attr("r", 2 * dna_attr.rect.width);
        })
        .on("mouseout", function(d) {
            dna_attr.vis.network_grp.select("#interaction-" + d.id).attr("fill", dna_attr.rect.colour_inner(d.id.split('n')[1]));
            dna_attr.vis.network_grp.select("#interaction-" + d.id).attr("r", dna_attr.rect.width);
        })

//    node.append("title")
//            .text(function (d) {
//                return d.id;
//            });

    simulation
            .nodes(graph.nodes)
            .on("tick", ticked);

    simulation.force("link")
            .links(graph.links);

    function ticked() {
        link
                .attr("x1", function (d) {
                    return d.source.x;
                })
                .attr("y1", function (d) {
                    return d.source.y;
                })
                .attr("x2", function (d) {
                    return d.target.x;
                })
                .attr("y2", function (d) {
                    return d.target.y;
                });

        node
                .attr("cx", function (d) {
                    return d.x;
                })
                .attr("cy", function (d) {
                    return d.y;
                });
    }


    function dragstarted(d) {
        if (!d3.event.active)
            simulation.alphaTarget(0.3).restart();
        d.fx = d.x;
        d.fy = d.y;
    }

    function dragged(d) {
        d.fx = d3.event.x;
        d.fy = d3.event.y;
    }

    function dragended(d) {
        if (!d3.event.active)
            simulation.alphaTarget(0);
        d.fx = null;
        d.fy = null;
    }

}


/**
 * Setup svg
 */
var setup_strand_svg = function () {
    var margin = {top: 0, right: 100, bottom: 50, left: 20};

    var svg = d3.select("svg#strandvis"),
            width = +svg.attr("width") - margin.left - margin.right,
            height = +svg.attr("height") - margin.top - margin.bottom,
            angles = d3.range(0, 2 * Math.PI, Math.PI / 200);

    // Remove all existing content
    svg.selectAll("*").remove();

    svg = svg.append("g")
        .attr("transform", "translate(" + margin.left +
              "," + margin.top + ")");

    dna_attr.vis.translation_x = (margin.left/2 + dna_attr.semi_circle.outer_radius);
    dna_attr.svg = svg;
    dna_attr.vis.width = width;
    dna_attr.vis.height = height;
    dna_attr.vis.margin = margin;
    dna_attr.angles = angles;
}


var draw_interaction_points = function() {

        dna_attr.vis.network_grp.selectAll("interaction-pts")
            .data(dna_attr.rect.node_points)
            .enter().append("circle")
            .attr('id', function(d) {
                return "interaction-n" + d.id;
        })
            .attr('class', 'interaction')
            .attr("fill", function(d) {
                return dna_attr.rect.colour_inner(d.id);
        })
            .attr("cx", function(d) {
                return d.x
        })
            .attr("cy", function(d) {
                return d.y;
        })
            .attr("r", dna_attr.rect.width)
        .attr("transform", "translate(" + dna_attr.vis.translation_x +
        "," + dna_attr.vis.height + ")rotate(-180)")
        .on("mouseover", function(d) {

            dna_attr.fdg_svg.select("#fdg-n" + d.id).attr("fill", "black");
            dna_attr.fdg_svg.select("#fdg-n" + d.id).attr("opacity", 1);
            dna_attr.fdg_svg.select("#fdg-n" + d.id).attr("r", 2 * dna_attr.vis.fdg_rad);
        })
        .on("mouseout", function(d) {
            dna_attr.fdg_svg.selectAll(".nodes").attr("opacity", 1);
            dna_attr.fdg_svg.select("#fdg-n" + d.id).attr("fill", dna_attr.rect.colour_inner(d.id));
            dna_attr.fdg_svg.select("#fdg-n" + d.id).attr("r", dna_attr.vis.fdg_rad);
        })
}



/**
 * Draws the looping strand between the nodes.
 */
var draw_dna_len = function (nodes) {
    var line_data = [];
    dna_attr.rect.node_points = new Array();
    var inner_rad = dna_attr.semi_circle.inner_radius;

    // Add the line to the first node
    if (dna_attr.semi_circle.rad_types != "normal") {
        var x = 2*dna_attr.semi_circle.outer_radius -
                dna_attr.semi_circle.detailed_radi[nodes[0].num_edges];
        var y = 0;
        line_data.push({x: x, y: y});
    } else {
        var x = 2 * dna_attr.semi_circle.outer_radius;
        var y = 0;
        line_data.push({x: x, y: y});
    }

    for (var n = 0; n < nodes.length - 1; n++) {
        var node1 = nodes[n];
        var node2 = nodes[n + 1];
        if (dna_attr.semi_circle.rad_types == "normal") {
            var mid_radius = dna_attr.semi_circle.outer_radius
            var size =  mid_radius + (((nodes[n + 1].start - nodes[n].start) / data.max_dist) * dna_attr.dna_strand.external_dist);
            line_data = calc_loop_interpolation_points(node1.end_angle, node2.start_angle,
                inner_rad, mid_radius, size, line_data, n);
        } else {
        var     start2 = nodes[n + 1].start;
        var     start_angle2 = node2.start_angle;
        var    mid_rad2 = dna_attr.semi_circle.detailed_radi[node2.num_edges];

        var mid_rad1 = dna_attr.semi_circle.detailed_radi[node1.num_edges];//dna_attr.semi_circle.outer_radius

        if (mid_rad1 == undefined || isNaN(mid_rad1)) {
            mid_rad1 = dna_attr.semi_circle.outer_radius;
        }
        if (mid_rad2 == undefined || isNaN(mid_rad2)) {
            mid_rad2 = dna_attr.semi_circle.outer_radius;
        }

        var size =  mid_rad1 + (((start2 - nodes[n].start) / data.max_dist) * dna_attr.dna_strand.external_dist);
        if (mid_rad2 > mid_rad1) {
            size =  mid_rad2 + (((start2 - nodes[n].start) / data.max_dist) * dna_attr.dna_strand.external_dist);
        }
        line_data = calc_loop_interpolation_points_diff_rads(node1.end_angle, start_angle2,
                mid_rad1 - 10, mid_rad2 - 10, mid_rad1, mid_rad2 , size, line_data, n);
            }
    }
    // Add the last line to the last node

        var x = -2 *dna_attr.semi_circle.outer_radius;
        var y = 0;
        line_data.push({x: x, y: y});

 var last_node_angle = nodes[nodes.length - 1].end_angle;
    dna_attr.rect.node_points.push({x: inner_rad * Math.cos(last_node_angle), y: inner_rad * Math.sin(last_node_angle), id: nodes.length - 1});

    draw_line(line_data, dna_attr.dna_strand.dist_stroke_colour);

}

/**
 * Calculates the interpolation point for the join between two nodes.
 */
var calc_loop_interpolation_points = function (angle1, angle2, inner_rad, mid_rad, outer_rad, line_data, node_num) {
    var top_rad = (mid_rad + outer_rad) / 2;
    var small_rad = (mid_rad + inner_rad) / 2;
    var mid_angle = (angle2 + angle1) / 2;
    var start_mid = (angle1 + mid_angle) / 2;
    var end_mid = (angle2 + mid_angle) /2;
    inner_pt_1 = {x: inner_rad * Math.cos(angle1), y: inner_rad * Math.sin(angle1)};
    inner_pt_2 = {x: inner_rad * Math.cos(angle2), y: inner_rad * Math.sin(angle2)};
    small_pt_1 = {x: small_rad * Math.cos(angle1), y: small_rad * Math.sin(angle1)};
    small_pt_2 = {x: small_rad * Math.cos(angle2), y: small_rad * Math.sin(angle2)};
    mid_pt_1 = {x: mid_rad * Math.cos(start_mid), y: mid_rad * Math.sin(start_mid)};
    mid_pt_2 = {x: mid_rad * Math.cos(end_mid), y: mid_rad * Math.sin(end_mid)};
    top_loop_curve1 = {x: top_rad * Math.cos(start_mid), y: top_rad * Math.sin(start_mid)};
    top_loop_curve2 = {x: top_rad * Math.cos(end_mid), y: top_rad * Math.sin(end_mid)};
    top_loop_center = {x: outer_rad * Math.cos(mid_angle), y: outer_rad * Math.sin(mid_angle)};

    if (node_num == 0) {
        line_data.push(inner_pt_1);
    }
     dna_attr.rect.node_points.push({x: inner_rad * Math.cos(angle1), y: inner_rad * Math.sin(angle1), id: node_num});

        line_data.push(mid_pt_1);
        line_data.push(top_loop_curve1);
        line_data.push(top_loop_center);
        line_data.push(top_loop_curve2);
        line_data.push(mid_pt_2);

   // Draw a rect at each "interaction point"
    line_data.push(inner_pt_2);
    return line_data;
}
///**
// * Calculates the interpolation point for the join between two nodes.
// */
var calc_loop_interpolation_points_diff_rads = function (angle1, angle2, inner_rad1, inner_rad2, mid_rad1, mid_rad2, outer_rad, line_data, node_num) {
    var top_rad = (mid_rad2 + outer_rad) / 2;
    if (mid_rad1 > mid_rad2) {
        top_rad = (mid_rad1 + outer_rad) / 2;
    }
    var small_rad = (mid_rad1 + inner_rad1) / 2;
    if (mid_rad1 > mid_rad2) {
        small_rad = (mid_rad2 + inner_rad2) / 2;
    }

    var mid_angle = (angle2 + angle1) / 2;
    var start_mid = (angle1 + mid_angle) / 2;
    var end_mid = (angle2 + mid_angle) /2;
    inner_pt_1 = {x: inner_rad1 * Math.cos(angle1), y: inner_rad1 * Math.sin(angle1)};
    inner_pt_2 = {x: inner_rad2 * Math.cos(angle2), y: inner_rad2 * Math.sin(angle2)};
    small_pt_1 = {x: small_rad * Math.cos(angle1), y: small_rad * Math.sin(angle1)};
    small_pt_2 = {x: small_rad * Math.cos(angle2), y: small_rad * Math.sin(angle2)};
    mid_pt_1 = {x: mid_rad1 * Math.cos(start_mid), y: mid_rad1 * Math.sin(start_mid)};
    mid_pt_2 = {x: mid_rad2 * Math.cos(end_mid), y: mid_rad2 * Math.sin(end_mid)};
    top_loop_curve1 = {x: top_rad * Math.cos(start_mid), y: top_rad * Math.sin(start_mid)};
    top_loop_curve2 = {x: top_rad * Math.cos(end_mid), y: top_rad * Math.sin(end_mid)};
    top_loop_center = {x: outer_rad * Math.cos(mid_angle), y: outer_rad * Math.sin(mid_angle)};

    if (node_num == 0) {
        line_data.push(inner_pt_1);
    }
     dna_attr.rect.node_points.push({x: inner_rad1 * Math.cos(angle1), y: inner_rad1 * Math.sin(angle1), id: node_num});

        //line_data.push(mid_pt_1);
        line_data.push(top_loop_curve1);
        line_data.push(top_loop_center);
        line_data.push(top_loop_curve2);
        //line_data.push(mid_pt_2);

   // Draw a rect at each "interaction point"
    line_data.push(inner_pt_2);
    return line_data;
}


/**
 *
 * Simple line function that determins the interpolation.
 */
var line_function = d3.line()
        .x(function (d) {
            return d.x;
        })
        .y(function (d) {
            return d.y;
        })
        .curve(d3.curveCardinal);


/**
 *
 * Draw the DNA strand based on the points added above.
 */
var draw_line = function (line_data, colour) {
    var svg = dna_attr.vis.network_grp;
    svg.append("path")
            .attr("d", line_function(line_data))
            .attr("fill", "none")
            .attr("stroke", colour)
            .attr("stroke-width", dna_attr.dna_strand.stroke_width)
            .attr("transform", "translate(" + dna_attr.vis.translation_x + "," + dna_attr.vis.height + ")rotate(-180)");

}


/**
 * =============================================================================
 *                        Draw the networks around that network.
 * =============================================================================
 */
var run_network_update = function() {
    setup_network_scales();
}


var setup_network_scales = function() {
    var start = processed_data.curr_start;
    var end = processed_data.curr_end;
    var network_start = processed_data.curr_network_start;
    var network_end = processed_data.curr_network_end;
    var network_size = dna_attr.semi_circle.outer_radius + (2 * dna_attr.main_strand.tail_length);
    var position = (network_start - start) / (end - start);

    // the network.
    // Make two scales one for the left hand side and one for the right side.
    var x_left = d3.scaleLinear()
            .domain([start - 1, network_start])
            .range([0, dna_attr.vis.width * position - dna_attr.main_strand.tail_length]),
        x_right = d3.scaleLinear()
            .domain([network_end, end + 1])
            .range([dna_attr.main_strand.tail_length, dna_attr.vis.width - (dna_attr.vis.width * position)]);
            y = d3.scaleLinear()
            .domain([0, 2])
            .range([0, 50]);

    var max_ticks = 10;
    var left_ticks = (position * 10);
    var right_ticks = (max_ticks - left_ticks);

    var xAxis_left = d3.axisBottom(dna_attr.vis.scale.x_left).tickSize(0).ticks(left_ticks),
            xAxis_right = d3.axisBottom(dna_attr.vis.scale.x_right).tickSize(0).ticks(right_ticks);

    dna_attr.svg.selectAll(".left-grp").remove();
    dna_attr.svg.selectAll(".right-grp").remove();

    var left = dna_attr.svg.append("g")
            .attr("class", "left-grp")
            .attr("transform", "translate(" + dna_attr.vis.margin.left +
                    "," + dna_attr.vis.margin.top + ")");

    var network_grp = dna_attr.svg.append("g")
            .attr("class", "network-grp")
            .attr("transform", "translate(" + dna_attr.vis.width * position +
                    dna_attr.main_strand.tail_length +
                    "," + dna_attr.vis.margin.top + ")");

    var right_offset = dna_attr.vis.width * position + network_size;
    var right = dna_attr.svg.append("g")
            .attr("class", "right-grp")
            .attr("transform", "translate(" + right_offset +
                    "," + dna_attr.vis.margin.top + ")");


    dna_attr.vis.xAxis_left = xAxis_left;
    dna_attr.vis.xAxis_right = xAxis_right;
    dna_attr.vis.scale.x_left = x_left;
    dna_attr.vis.scale.x_right = x_right;
    dna_attr.vis.scale.y = y;
    dna_attr.vis.network_grp = network_grp;
    dna_attr.vis.left_grp = left;
    dna_attr.vis.right_grp = right;

}


var draw_network_axis = function () {
    dna_attr.vis.left_grp.append("g")
            .attr("class", "axis x-axis")
            .attr("transform", "translate(0," + dna_attr.vis.height + ")")
            .call(dna_attr.vis.xAxis_left);

    dna_attr.vis.right_grp.append("g")
            .attr("class", "axis x-axis")
            .attr("transform", "translate(0," + dna_attr.vis.height +
                    ")")
            .call(dna_attr.vis.xAxis_right);
}


var draw_other_networks = function() {
    var networks = filter_on_networks();
    // Draw the first half
    draw_network_positions(filter_on_position(processed_data.curr_start,
                processed_data.curr_network_start), dna_attr.vis.left_grp,
                        dna_attr.vis.scale.x_left);

    draw_network_positions(filter_on_position(processed_data.curr_network_end,
        processed_data.curr_end), dna_attr.vis.right_grp,
                dna_attr.vis.scale.x_right);

    // Remove the network filter
        processed_data.type_dimension.filterAll();

}

var draw_network_positions = function(network_data, group, scale) {
        var networks = group.append("g");
        networks.selectAll("network")
            .data(network_data)
            .enter().append('rect')
            .attr('id', function (d, i) {

                    // assign an arbitary id for networks.
                    return "networkLine-" + i;

            })
            .attr('class', 'network')
            .attr("fill", function (d) {
                return genome_attr.vis.colour(d.chrom);
            })
            .style("opacity", genome_attr.vis.rect.opacity_lrg)
            .attr("x", function (d) {
                return scale(d.start);
            })
            .attr("y", function (d) {
                return dna_attr.vis.height - 12;
            })
            .attr("width", function (d) {

                    return genome_attr.vis.rect.width_lrg;

            })
            .attr("height", genome_attr.vis.rect.height_lrg)
            .on("mouseover", function (d) {
                network_tooltip.transition()
                        .duration(200)
                        .style("opacity", .9);
                 network_tooltip.html("chr: " + d.chrom + "<br/>" + "start: " + d.start + "<br/>" + "end: " + d.end)
                        .style("left", (d3.event.pageX) + "px")
                        .style("top", (d3.event.pageY + 28) + "px");

            })
            .on("mouseout", function (d) {
                network_tooltip.transition()
                        .duration(500)
                        .style("opacity", 0);
            })
            .on("click", function (d) {
                // Remove the tooltip
                network_tooltip.transition()
                        .duration(500)
                        .style("opacity", 0);
                    // Want to update the visualisation below
                    processed_data.curr_network_start = +d.start;
                    processed_data.curr_network_end = +d.end;
                    run_dna_visualisation();
                    //getNetworkByChromAndPosition(d.chrom, d.start, d.end);

            });
}

var draw_tails = function () {
    // Draw the initial tail
    dna_attr.vis.network_grp.append("line")
            .attr("x1", - dna_attr.main_strand.tail_length/ 2 - 10)
            .attr("x2", dna_attr.main_strand.tail_length/ 2)
            .attr("y1", dna_attr.vis.height - 1)
            .attr("y2", dna_attr.vis.height - 1)
            .attr("stroke", dna_attr.main_strand.stroke_colour)
            .attr("stroke-width", dna_attr.main_strand.stroke_width)
            .attr("transform", "translate(0," + dna_attr.vis.height + ")")

    // Daw the ending tail
    dna_attr.vis.network_grp.append("line")
            .attr("x1", dna_attr.semi_circle.inner_radius * 2 + dna_attr.main_strand.tail_length/ 2)
            .attr("x2", dna_attr.semi_circle.inner_radius * 2 + (2 * dna_attr.main_strand.tail_length))
            .attr("y1", dna_attr.vis.height - 1)
            .attr("y2", dna_attr.vis.height - 1)
            .attr("stroke", dna_attr.main_strand.stroke_colour)
            .attr("stroke-width", dna_attr.main_strand.stroke_width)
            .attr("transform", "translate(0," + dna_attr.vis.height + ")")
}





var draw_nodes = function (nodes) {
    var pi = Math.PI;
    var full_circle = pi;
    var region_len = full_circle / (nodes.length - 1);
    for (var i in nodes) {// node in nodes) {
        var start_angle = (i * region_len);
        var end_angle = (i * region_len);
        //draw_node_rect(start_angle, end_angle);

        // Save the start and end angle to the node
        nodes[i].start_angle = start_angle;
        nodes[i].end_angle = end_angle;
    }
}

var diff_dna_view = function() {
    if (dna_attr.semi_circle.rad_types == "normal") {
        dna_attr.semi_circle.rad_types = "multi";
    } else {
       dna_attr.semi_circle.rad_types = "normal"
    }
    run_dna_visualisation();
}
